/* gmessage.c
 * 
 *  gmessage, a xmessage clone written with gtk+
 * 
 *  Copyright (c) 1999 Daniel Adler 'plex/TEK! neoscientists'
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */
#include <gtk/gtk.h>
#include <stdio.h>

#define MAX_BUT	16
#define TEXTBUF_SIZE 64*1024
#define LINEBUF_SIZE 1024

int retcode;

gchar *help_text[] = 
{
"usage: gmessage [-options] message ...\n",
"\n",
"where options are:\n",
"  -help\t\t\t\t\tprint this text :)\n",
"  -version\t\t\t\tprint version\n",
"  -file filename\t\t\tfile to read message from, \"-\" for stdin\n",
"  -frametext string\t\t\tframe text\n",
"  -buttons string\t\t\tcomma-seperated list of buttons (with exit-codes 1..n )\n",
"  -center\t\t\t\tpop up at center of screen\n",
"  -nearmouse\t\t\t\tpop up near the mouse cursor\n",
"  -justify left|right|center|fill\tjustifies the text\n",
"  -timeout secs\t\t\t\texit with code 0 after \"secs\" seconds\n",
"  -print\t\t\t\tprint the button label when selected\n",
"  -geometry geom\t\t\tset window geometry (usage:wxh+x+y)\n",
"\n",
NULL
};

gchar *version_text[] =
{
"gmessage v0.2  (c) 1999 by Daniel 'plex/TEK' Adler  <dadler@gwdg.de>\n\n",
NULL
};

gboolean print_on_exit = FALSE;
gchar *text;
guint textlen;

gint
timeout_quit(gpointer userdata)
{
	gtk_main_quit();
}

void
winquit(GtkWidget *widget, GdkEventAny *event, gpointer userdata)
{
	gtk_main_quit();
}

void
print_list(gchar **text)
{
	guint j;	
	j = 0;
	while(text[j] != NULL)
	{
		g_print(text[j++]);
	}
}
	
void
version()
{
	print_list(version_text);
	exit(retcode);
}

void
usage()
{
	print_list(help_text);
	exit(retcode);
}
		
void
but_clicked(GtkButton *button, gpointer userdata)
{
	gint *ptr;
	ptr = userdata;
	if (print_on_exit == TRUE)
	{
		g_print(userdata);
		g_print("\n");
	}
	
	gtk_main_quit();
	retcode = *ptr;
}

void
append_text(gchar *s)
{
	gint len;
	len = strlen(s);
	if (textlen+len > TEXTBUF_SIZE)
	{
		g_error("text buffer too small");
		exit(-1);
	}

	if (textlen != 0)
	{
		/* append text (insert space) */
		text[textlen] = ' ';
		textlen++;
	}
	
	strcpy(text+textlen,s);
	textlen += len;
}
	
int
main(int argc, char *argv[])
{
	gint i;
	gint win_x = -1,win_y = -1;
	guint win_width = 0,win_height = 0;
	gchar *frametext = NULL;
	gchar *defaultbut = NULL;
	gchar *line;
	gint timeout = -1;

	
	GList *buttons = NULL;
	gchar **butlabelarray = NULL;
	GtkWidget *win, *vbox, *frame, *scrollwin, *vp, *label, *hbb, *b;
	GtkRequisition req;
	GtkWindowPosition winpos = GTK_WIN_POS_NONE;
	GtkJustification textjustify = GTK_JUSTIFY_CENTER;
	
	retcode = 1;
	
	gtk_init(&argc,&argv);

	if (argc == 1)
	{
		usage();
	}

	line = g_malloc(LINEBUF_SIZE);
	text = g_malloc(TEXTBUF_SIZE);

	text[0] = '\0';
	textlen = 0;
	
	for (i=1;i<argc;i++)
	{
		if (argv[i][0] == '-')
		{
			/* options */

			gchar *option;

			option = argv[i]+1;
			
			/* single options */
			
			if (strcmp(option,"help") == 0)
			{
				usage();
			}
			else if (strcmp(option,"version") == 0)
			{
				version();
			}
			else if (strcmp(option,"nearmouse") == 0)
			{
				winpos = GTK_WIN_POS_MOUSE;
			}
			else if (strcmp(option,"center") == 0)
			{
				winpos = GTK_WIN_POS_CENTER;
			}
			else if (strcmp(option,"print") == 0)
			{
				print_on_exit = TRUE;
			}

			/* multi options */
		
			else if (strcmp(option,"geometry") == 0)
			{
				i++;
				XParseGeometry(argv[i],&win_x, &win_y, &win_width, &win_height);
			}	
			else if (strcmp(option,"frametext") == 0)
			{
				i++;
				frametext = argv[i];
			}
			else if (strcmp(option,"buttons") == 0)
			{
				gchar *list;

				i++;
				list = argv[i];

				butlabelarray = g_strsplit(list,",",MAX_BUT);
			}
			else if(strcmp(option, "default" ) == 0)
			{
				i++;
				defaultbut = argv[i];
			}
			else if(strcmp(option, "file" ) == 0)
			{
				FILE *file;

				i++;

				if (strcmp(argv[i],"-") == 0)
				{
					/* stdin */
					file = stdin;
				}
				else
				{
					file = fopen(argv[i],"r");
				}
				if (file == NULL)
				{
					g_error("can't open file %s",argv[i+1]);
				}
				else
				{
					/* load file in text buffer */
					gchar *a;
					a = fgets(line,1024,file);
					while(a != NULL)
					{
						append_text(a);
						a = fgets(line,1024,file);
					}
					fclose(file);
				}
			}
			else if(strcmp(option, "timeout") == 0)
			{
				i++;
				timeout = atoi(argv[i]);
			}
			else if (strcmp(option,"justify") == 0)
			{
				i++;
				option = argv[i];
				if (strcmp(option,"left") == 0)
					textjustify = GTK_JUSTIFY_LEFT;
			
				else if (strcmp(option,"right") == 0)
					textjustify = GTK_JUSTIFY_RIGHT;
				
				else if (strcmp(option,"center") == 0)
					textjustify = GTK_JUSTIFY_CENTER;
				 
				else if (strcmp(option,"fill") == 0)
					textjustify = GTK_JUSTIFY_FILL;	 
			}
		}
		else
		{
			/* message */
			append_text(argv[i]);
		}
	}
	
	g_free(line);

	/* --- gui build --- */
	
	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_position(GTK_WINDOW(win),winpos);
	
	gtk_signal_connect(GTK_OBJECT(win), "destroy_event", winquit, NULL);
	gtk_signal_connect(GTK_OBJECT(win), "delete_event", winquit, NULL);
	
	gtk_container_set_border_width(GTK_CONTAINER(win),5);
	vbox = gtk_vbox_new(0,0);
	
	frame = gtk_frame_new(frametext);
	scrollwin = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	vp = gtk_viewport_new(NULL,NULL);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(vp), GTK_SHADOW_NONE);

	label = gtk_label_new(text);

	gtk_label_set_justify(GTK_LABEL(label), textjustify);
	
	gtk_container_add(GTK_CONTAINER(vp), label);
	gtk_widget_show(label);
	
	gtk_container_add(GTK_CONTAINER(scrollwin), vp);
	gtk_widget_show(vp);
	
	gtk_container_set_border_width(GTK_CONTAINER(scrollwin),3);
	gtk_widget_show(scrollwin);
	
	gtk_container_add(GTK_CONTAINER(frame), scrollwin);
	
	gtk_box_pack_start(GTK_BOX(vbox),frame,1,1,0);
	gtk_widget_show(frame);
	
	hbb = gtk_hbutton_box_new();
	gtk_container_set_border_width(GTK_CONTAINER(hbb),5);
	if (butlabelarray == 0)
	{
		gchar *label = "okay";
		gint *ptr;
		ptr = g_new(gint,1);
		*ptr = 0;
		
		b = gtk_button_new_with_label(label);
		gtk_signal_connect(GTK_OBJECT(b),"clicked",but_clicked,label);
		gtk_widget_show(b);
		gtk_container_add(GTK_CONTAINER(hbb),b);
	}
	else
	{
		int j;

		j=0;
		while ((butlabelarray[j] != NULL) && (j < MAX_BUT))
		{
			gint *ptr;
		
			ptr = g_new(gint,1);
			*ptr = j+1;
			
			b = gtk_button_new_with_label(butlabelarray[j]);
			gtk_signal_connect(GTK_OBJECT(b),"clicked",but_clicked,butlabelarray[j]);
			gtk_widget_show(b);
			gtk_container_add(GTK_CONTAINER(hbb),b);
	
			if ((defaultbut != NULL) &&
				(strcmp(butlabelarray[j],defaultbut) == 0) )
			{
				
			}

			
			j++;
		}
	}
		
	gtk_widget_show(hbb);
	gtk_box_pack_start(GTK_BOX(vbox),hbb,0,1,0); 
	gtk_widget_show(vbox),

	gtk_container_add(GTK_CONTAINER(win),vbox);

	if (win_width != -1)
	{
		gtk_window_set_default_size(GTK_WINDOW(win), win_width, win_height);
	}
	if (win_x != -1)
	{
		gtk_widget_set_uposition(win, win_x, win_y);
	}
	
	gtk_widget_show(win);
	
	if (timeout != -1)
	{
		gtk_timeout_add(1000*timeout, timeout_quit, NULL);
	}

	gtk_main();
	
	return (retcode);
	
}
